#!/usr/bin/env python3

TOKEN = 'your-token'

import logging
import random

from telegram import ParseMode, ReplyKeyboardMarkup, ReplyKeyboardRemove, User
from telegram.ext import Updater, CommandHandler, MessageHandler, Filters

import permissions
import gpioman
import useradmin
from utilities import *

# Enable logging
logging.basicConfig(
        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        level=logging.INFO
    )

logger = logging.getLogger(__name__)

## The class that manages the bot
class SgabelliBot(SharedTexts):

    # LAMPS to command (command, GPIO number, description)
    LAMPS = [
        ('lampadablu', 5, 'Lampada blu'),
        ('lampadanera', 6, 'Lampada nera')
    ]

    # Generic sentences for start and help
    HELLO_STRANGER = em('Ciao! Non mi pare di conoscerti! :thinking_face: \n' +
            'Però se sei un ospite, contatta _tu-sai-chi_ :wink:')
    WELCOME_BACK = em('Bentornato! :nerd_face:')
    HELP_TEXT = 'Questo è un bot ad uso privato. Però se sai chi lo ' + \
            'gestisce, puoi contattarlo e chiedere che ti abiliti.'

    # Generic sentences for turning on/off or status
    ON_TEXT = em('accesa :bulb:')
    OFF_TEXT = em('spenta :electric_plug:')

    OK_ON = em('Ok, lampada accesa :bulb:')
    OK_OFF = em('Ok, lampada spenta :electric_plug:')

    # Buttons
    STATUS_BUTTON = em('Stato :question:')
    ALL_ON = em('Accendi tutto :bulb:')
    ALL_OFF = em('Spegni tutto :electric_plug:')

    # Words to turn on or off the lamps for command /{lampname}
    CMD_ON = ['on', 'accendi', '1']
    CMD_OFF = ['off', 'spegni', '0']

    ## The constructor, initializes the permissions and the GPIO
    def __init__(self):
        self.permissions = permissions.Permissions()
        self.lampButtons = []

        for lamp in self.LAMPS:
            gpioman.setOutput(lamp[1])
            self.lampButtons.append(lamp[2])

    ## Run the bot
    def run(self):
        updater = Updater(TOKEN)

        dp = updater.dispatcher
        dp.add_error_handler(self.error)

        dp.add_handler(CommandHandler('start', self.start))
        dp.add_handler(CommandHandler('help', self.help))
        dp.add_handler(CommandHandler('stato', self.status))
        dp.add_handler(CommandHandler('accendi', self.turnOnAll))
        dp.add_handler(CommandHandler('spegni', self.turnOffAll))
        dp.add_handler(useradmin.UserAdmin(self.permissions,
                self._replyKeyboard))

        for lamp in self.LAMPS:
            dp.add_handler(LampHandler(self.lamp, lamp))

        # This must be the last one, because handlers are a sort of chain,
        # therefore setting this one before other can prevent them for running
        dp.add_handler(MessageHandler(Filters.text, self.messages))

        # Blocks until CTRL-C is called
        updater.start_polling()
        updater.idle()

    ## Log Errors caused by Updates
    def error(self, bot, update, error):
        logger.warning('Update "%s" caused error "%s"', update, error)

    ## Handle the command start
    def start(self, bot, update):
        if self.permissions.isKnown(update.effective_user):
            self._replyKeyboard(update, self.WELCOME_BACK)
        else:
            update.message.reply_text(self.HELLO_STRANGER,
                    parse_mode=ParseMode.MARKDOWN,
                    reply_markup=ReplyKeyboardRemove())

    ## Handle the command help
    def help(self, bot, update):
        update.message.reply_text(self.HELP_TEXT)

    ## Handle the command status
    def status(self, bot, update):
        if self._notAuthorized(update):
            return

        reply = ''

        for lamp in self.LAMPS:
            tmp = self.ON_TEXT if gpioman.getStatus(lamp[1]) else \
                    self.OFF_TEXT
            reply += lamp[2] + ': ' + tmp + '\n'

        self._replyKeyboard(update, reply)

    ## Turn on all lamps handler
    def turnOnAll(self, bot, update):
        self._turnAll(bot, update, True)

    ## Turn off all lamps handler
    def turnOffAll(self, bot, update):
        self._turnAll(bot, update, False)

    ## Handle a lamp command
    def lamp(self, bot, update, lamp, args):
        if self._notAuthorized(update):
            return

        toggle = False
        if len(args) > 0:
            desired = args[0].lower()
            if desired in self.CMD_OFF:
                new = False
            elif desired in self.CMD_ON:
                new = True
            else:
                toggle = True
        else:
            toggle = True

        if toggle:
            new = gpioman.toggle(lamp[1])
        else:
            gpioman.setStatus(lamp[1], new)

        reply = self.OK_ON if new else self.OK_OFF
        self._replyKeyboard(update, reply)

    ## Handle messages, which are used for buttons
    def messages(self, bot, update):
        if self._notAuthorized(update):
            return

        text = update.message.text
        if text == self.STATUS_BUTTON:
            self.status(bot, update)
        elif text == self.ALL_ON:
            self._turnAll(bot, update, True)
        elif text == self.ALL_OFF:
            self._turnAll(bot, update, False)
        else:
            for lamp in self.LAMPS:
                if text == lamp[2]:
                    self.lamp(bot, update, lamp, [])
                    break

    ## Check if a user is not authorized and in case send a get off message
    #
    # We use inverted logic so that we have to negate it only there, then the
    # functions that call this function do not have to invert always the outcome
    # of this function
    def _notAuthorized(self, update):
        if self.permissions.isKnown(update.effective_user):
            return False
        else:
            update.message.reply_text(random.choice(self.UNAUTHORIZED),
                    reply_markup=ReplyKeyboardRemove())
            return True

    ## Turn on or off all lamps (helper for other methods)
    def _turnAll(self, bot, update, status):
        if self._notAuthorized(update):
            return

        for lamp in self.LAMPS:
            new = gpioman.setStatus(lamp[1], status)

        self._replyKeyboard(update, random.choice(self.SUCCESS))

    ## Reply to a message and send the default keyboard
    def _replyKeyboard(self, update, text, **kwargs):
        keyboard = ReplyKeyboardMarkup([
                self.lampButtons,
                [self.ALL_ON, self.STATUS_BUTTON, self.ALL_OFF]
            ], resize_keyboard=True)
        if update:
            update.message.reply_text(text, reply_markup=keyboard)
        elif kwargs['bot'] and kwargs['chatId']:
            kwargs['bot'].send_message(chat_id=kwargs['chatId'], text=text,
                    reply_markup=keyboard)

## A class used to call the real lamp handler with lamp as additional parameter
class LampHandler(CommandHandler):
    def __init__(self, callback, lamp):
        CommandHandler.__init__(self, lamp[0], callback, pass_args=True)
        self.lamp = lamp

    def collect_optional_args(self, dispatcher, update):
        args = CommandHandler.collect_optional_args(self, dispatcher, update)
        args['lamp'] = self.lamp
        return args

if __name__ == '__main__':
    gpioman.start()
    b = SgabelliBot()
    b.run()
    gpioman.stop()
